/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"
#include "dlfcn.h"

/* code for dynamically loading transports */

void dso_load(xmlnode x)
{
    void (*etherx_receive)(epacket e);
    void (*etherx_setup)(void *send, void *receive, void *alias, char *namespace);
    void *tp_so;
    char *defso;

    jdebug(ZONE,"dso_load: %s",xmlnode_get_attrib(x,"dso"));

    defso = xmlnode_get_attrib(x,"dso");
    tp_so = dso_open(defso);
    if(tp_so == NULL)
    {
        jerror("etherxd","DSO load of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }

    etherx_receive = dso_sym(tp_so,"etherx_receive");
    if(etherx_receive == NULL)
    {
        jerror("etherxd","DSO sym of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }
    etherx_setup = dso_sym(tp_so,"_etherx_setup");
    if(etherx_setup == NULL)
    {
        jerror("etherxd","DSO sym of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }

    (etherx_setup)(manager_send, etherx_receive, dso_alias, xmlnode_get_attrib(x, "namespace")); /* tell libetherx stuff */
    (etherx_receive)(NULL); /* tell transport to start up */

    registry_handle(xmlnode_get_attrib(x,"id"),xmlnode_get_attrib(x,"namespace"),etherx_receive,NULL);
}

void *dso_open(char *so)
{
    char *dlerr;
    void *so_h;

    if(so == NULL)
        return NULL;

    so_h = dlopen(so,RTLD_LAZY);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        jdebug(ZONE,"dso_open(%s) failed: %s",so, dlerr);
        return NULL;
    }

    return so_h;
}

void *dso_sym(void *so, char *func)
{
    char *dlerr;
    void *func_h;

    if(so == NULL || func == NULL)
        return NULL;

    func_h = dlsym(so,func);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        jdebug(ZONE,"dso_sym(%s) failed: %s",func, dlerr);
        return NULL;
    }

    return func_h;
}

void dso_alias(void (*handler)(epacket e), char *host, char *namespace)
{
    edb d;

    d = db(CMD_SET, host, namespace);
    d->dso_handler = handler;
    d->type = DB_TRANSPORT;
}

