/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <jabber/jlib.h>
#include "etherx.h"

#define CMD_SET 0
#define CMD_GET 1
#define CMD_DEL 2
 
#define DB_CONNECTION 0
#define DB_TRANSPORT 1
#define DB_ERROR 2

extern xmlnode config;

struct registry_t
{
    char *id;
    char *ns;
    void (*etherx_receive)(epacket e);
    thread t;
    xmlnode reg;
    struct registry_t *next;
};

void registry_include(xmlnode x);
void registry_load(char *rfile);
void registry_handle(char *id, char *ns, void (*etherx_receive)(epacket e), thread t);
void registry_send(struct registry_t *r, xmlnode x);
void registry_scan();
struct registry_t *registry_lookup(char *id, char *ns);

typedef struct edb_struct
{
    int type, count;
    char *host;
    char *namespace;
    void (*dso_handler)(epacket e); /* used if it's a dso transport */
    thread t; /* used if it's an xml stream */
    struct edb_struct *next;
} _edb, *edb;
 
edb db(int cmd, char *host, char *namespace);

void net_init();
void net_init_listen(xmlnode x);
void net_negotiate(thread t, xmlnode in, int type);
void net_onConnect(thread t, int sock, struct sockaddr_in sa);
void net_onNode(thread t, xmlnode in, int type);
void net_write(thread t, epacket e);
void net_cleanup(void *arg);
void net_error(thread t, tstream_buffer buffer);

void manager_send(epacket e);
void manager_deliver(edb d, epacket e);

/* connection management */
void transport_onNode(thread t, xmlnode in, int type); /* from socket transports */
void transport_deliver_onNode(thread t, xmlnode in, int type); /* from connections to a transport */
void transport_deliver_cleanup(void *arg);
void transport_send(epacket e);
void transport_cleanup(void *arg);
int transport_secret(char *id, char *secret);

void dso_load(xmlnode x);
void *dso_open(char *so);
void *dso_sym(void *so, char *func);
void dso_alias(void (*handler)(epacket e), char *host, char *namespace);

void logger_init();
void logger_conn(char *ns, char *from, char *to, int count);
void logger_transport(char *ns, char *from, char *id, int count);
void logger_conn2(char *ns, char *to, int count);
void logger_transport2(char *ns, char *to, char *id, int count);
