/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

/* this is the intake feed from transports to deliver epackets */
void manager_send(epacket e)
{
    edb d;
    thread t;
    tstream ts;

    jdebug(ZONE,"manager_send()");

    d = db(CMD_GET,e->to,e->ns);
    if(d == NULL || d->type == DB_ERROR)
    { 
        if(e->type == EPACKET_ERROR)
        {
            jerror("etherxd","can't bounce a packet to %s from %s, dropping",e->to, e->from);
            xmlnode_free(e->x);
            return;
        }

        if(d != NULL && d->type == DB_ERROR)
        {
            jerror("etherxd","%s flagged as error",e->to);
            epacket_error(e,"delivery failure");
            manager_send(e);
            return;
        }

        /* create a special new connection! xmlstream can parse it, but we'll handle the buffer */
        t =  tstream_connect(5269, e->to, _xmlstream_main, net_error, net_onNode);
        d = db(CMD_SET,e->to,e->ns);
        d->t = t;
        d->type = DB_CONNECTION;
        ts = (tstream)(t->data);
        ts->data_aux5 = (void *)d;
    }

    manager_deliver(d, e);
}

/* sends epacket to the target db entry, encoding where needed for socket delivery */
void manager_deliver(edb d, epacket e)
{
    if(e == NULL)
        return;

    if(d == NULL)
    {
        jerror("etherxd", "invalid attempt to deliver");
        return;
    }

    d->count++;

    if(d->type == DB_TRANSPORT)
    {
        xmlnode_put_attrib(e->x, "etherx:to", e->to);
        xmlnode_put_attrib(e->x, "etherx:from", e->from);
        if(e->type == EPACKET_ERROR)
            xmlnode_put_attrib(e->x, "etherx:error", e->error);
    }else{
        xmlnode_hide_attrib(e->x, "etherx:to");
        xmlnode_hide_attrib(e->x, "etherx:from");
    }

    if(d->dso_handler != NULL)
    {
        jdebug(ZONE,"manager_send(DSO, %s)",d->host);
        (d->dso_handler)(e);
    }else if(d->type == DB_TRANSPORT){
        jdebug(ZONE,"manager_send(SOCKET_TP, %s)",d->host);
        xmlstream_write(d->t, e->x);
    }else{
        jdebug(ZONE,"manager_send(SOCKET_NET, %s)",d->host);
        net_write(d->t, e); /* our special handler */
    }
}
