/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

void net_init()
{
    int did = 0;
    xmlnode self, next;
    /* look at registry and start tlisteners on appropriate IPs */
    /* create entries for own IP's in connection arrays and initi arrays */
    /* return to main thread */

    jdebug(ZONE,"net_init()");    

    self = xmlnode_get_tag(config,"etherx");	    
    if(self != NULL)
    {           
        next = xmlnode_get_firstchild(self);
        while(next != NULL)
        {
            if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
            {
                did = 1;
                net_init_listen(next);
            }
            next = xmlnode_get_nextsibling(next);
        }
    }
    if(!did)
        tlisten_new(5269,NULL,net_onConnect,NULL);
}

void net_init_listen(xmlnode x)
{
    char *ip;
    xmlnode x2;

    jdebug(ZONE,"net_init_listen()");

    /* XXX track what was done for reconfiguration */

    x2 = xmlnode_get_firstchild(x);
    ip = xmlnode_get_data(x2);
    if(ip != NULL)
    {
        jdebug(ZONE,"net_init_listen: %s",ip);
        tlisten_new(5269,ip,net_onConnect,NULL);
    }
}

/* this is the callback on the listener thread whenever there is a new connect */
void net_onConnect(thread t, int sock, struct sockaddr_in sa)
{
    thread tnew;

    jdebug(ZONE,"connection established (fd: %d, ip: %s, port: %d)",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    tnew = xmlstream_new(sock, inet_ntoa(sa.sin_addr), (xmlstream_onNode)net_negotiate);
}

/* this is the callback on new xmlstreams, it checks/sets-up the stream
 * and determines if this is a normal connect or a new socket-based transport
 */
void net_negotiate(thread t, xmlnode in, int type)
{
    char *ns, *to;
    tstream ts = (tstream)(t->data);
    xmlnode x;
    edb d;

    if(type != XMLSTREAM_ROOT)
        return;
 
    jdebug(ZONE,"net_negotiate socket %d\n",ts->fd);
 
    ns = xmlnode_get_attrib(in, "xmlns");
    to = xmlnode_get_attrib(in, "to");
 
    if(type == XMLSTREAM_ERROR || ns == NULL || to == NULL)
    {
        xmlstream_write_header(t, xmlstream_header(NULL, NULL, NULL));
        if(type == XMLSTREAM_ERROR)
            xmlstream_error(t, "You didn't send valid XML.");
        else if(ns == NULL)
            xmlstream_error(t, "You didn't specify a default xmlns='' namespace.");
        else if(to == NULL)
            xmlstream_error(t, "You didn't send your to='host' attribute.");
        return;
    }
 
    if(j_strcmp(xmlnode_get_attrib(in, "xmlns:etherx"), ETHERX_NAMESPACE) == 0)
    { /* it's trying to be a transport */
        jdebug(ZONE,"negoitiated: new transport");
        x = xmlstream_header(ns, NULL, to);
        xmlnode_put_attrib(x, "xmlns:etherx",ETHERX_NAMESPACE);
        xmlstream_write_header(t, x);
        ts->data = (void *)transport_onNode; /* it's a transport, let them manage it */
        transport_onNode(t, in, type); /* let the transport handler init */
    }else{
        /* it's connecting to a transport, find and hook together */
        jdebug(ZONE,"negoitiated: incoming connection");
        d = db(CMD_GET,to,ns);
        xmlstream_write_header(t, xmlstream_header(ns, NULL, to));
        if(d == NULL || d->type != DB_TRANSPORT)
        {
            xmlstream_error(t, "This server doesn't understand your namespace.");
        }else{
            ts->data_aux5 = (void *)d; /* so that transport_deliver_onNode knows which transport it's tied to */
            ts->data = (void *)transport_deliver_onNode; /* automatically bridge this xmlstream to this transport */
            transport_deliver_onNode(t, in, type); /* let the transport deliver handler init */
        }
    }
}


/* this is a simple handler for net connections etherx has made to others */
void net_onNode(thread t, xmlnode in, int type)
{
    tstream ts = (tstream)(t->data);
    edb d = (edb)(ts->data_aux5);

    if(type == XMLSTREAM_INIT)
    {
        xmlstream_write_header(t, xmlstream_header(d->namespace, d->host, NULL));
        pth_cleanup_push(net_cleanup, (void *)t);
        return;
    }

    if(type == XMLSTREAM_ERROR)
    {
        xmlstream_error(t,"You sent invalid XML.");
        return;
    }

    if(type == XMLSTREAM_NODE)
    {
        if(j_strcmp(xmlnode_get_name(in),"stream:error") == 0)
        {
            jerror("etherxd","remote server XML Stream error: %s",xmlnode_get_data(xmlnode_get_firstchild(in)));
            pth_exit(NULL);
            return;
        }
        jwarn("etherxd","receiving data on connection to remote server where we shouldn't be");
    }
}

void net_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    edb d = (edb)(ts->data_aux5);

    jdebug(ZONE,"cleaning up remote server connection db entries");

    if(d != NULL)
    {
        logger_conn2(d->namespace, d->host, d->count);
        db(CMD_DEL,d->host,d->namespace);
    }
}

void net_write(thread t, epacket e)
{
    tstream_buffer b;
 
    jdebug(ZONE,"net_write");
 
    if(t == NULL || e == NULL)
        return;
 
    b = pmalloc(xmlnode_pool(e->x),sizeof(struct tstream_buffer_struct));
    b->data = xmlnode2str(e->x);
    b->data_sz = strlen(b->data);
    b->sent = 0;
    b->obj = (void *)e;
    b->p = xmlnode_pool(e->x);
 
    tstream_write_buffer(t, b);
}

void net_error(thread t, tstream_buffer buffer)
{
    /* the stream is closed and these are the epackets that didn't get sent */
    epacket e, eold = (epacket)(buffer->obj);
 
    jdebug(ZONE,"net_error");
 
    if(buffer != NULL && eold != NULL)
    {
        e = epacket_new(xmlnode_dup(eold->x),eold->to, eold->from, eold->ns);
        jerror("etherxd","bouncing data intended for %s",e->to);
        epacket_error(e,"delivery failure");
        manager_send(e);
    }
}
