/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */


#define ETHERX_NAMESPACE "http://etherx.jabber.org/"

typedef struct epacket_struct
{
    xmlnode x;
    char *to;
    char *from;
    char *ns;
    char *error;
    int type;
} _epacket, *epacket;

#define EPACKET_UNKNOWN -1
#define EPACKET_NORMAL 0 /* default */
#define EPACKET_REGISTRY 1 /* special flag that this is a registry packet */
#define EPACKET_ERROR 2 /* special flag that this is an error/bounced packet */
#define EPACKET_ALIAS 3 /* this is a special packet that identifies a host alias for a transport */
#define EPACKET_TRANSPORT 4 /* transport's identification phase */

/* public functions */
void etherx_send(epacket e); /* sends epacket to etherx */
void etherx_alias(char *host); /* adds host alias for this transport to receive incoming data on */
epacket epacket_new(xmlnode x, char *to, char *from, char *ns); /* if to/from are null it extracts them from x, sets type based on element name */
void epacket_error(epacket e, char *error); /* simply turns a normal epacket into an error one */

/* socket based transport */
int etherx_init(char *id, char *secret, char *host, char *namespace, void (*receive)(epacket e));

/* dso stuff for etherxd */
void _etherx_setup(void *send, void *receive, void *alias, char *namespace); /* called by etherxd from within each dso to setup etherxd internal pointers/data */

/* internal functions */
void _etherx_receive(thread t, xmlnode x, int flag); /* callback for xmlstream when doing socket connects */
void _etherx_cleanup(void *arg); /* attempts to reconnect to etherxd */
