/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <jabber/jlib.h>
#include "etherx.h"

/* code to handle epackets */

epacket epacket_new(xmlnode x, char *to, char *from, char *ns)
{
    epacket e;
    char *name;

    name = xmlnode_get_name(x);
    if(x == NULL || name == NULL)
        return NULL;

    e = pmalloc(xmlnode_pool(x), sizeof(_epacket));

    if(to == NULL)
        e->to = xmlnode_get_attrib(x, "etherx:to");
    else
        e->to = pstrdup(xmlnode_pool(x), to);

    if(from == NULL)
        e->from = xmlnode_get_attrib(x, "etherx:from");
    else
        e->from = pstrdup(xmlnode_pool(x), from);

    e->type = EPACKET_NORMAL;
    if(strstr(name,"etherx:") != NULL)
    {
        e->type = EPACKET_UNKNOWN;
        if(strcmp(name, "etherx:registry") == 0)
            e->type = EPACKET_REGISTRY;
        if(strcmp(name, "etherx:alias") == 0)
            e->type = EPACKET_ALIAS;
        if(strcmp(name, "etherx:transport") == 0)
            e->type = EPACKET_TRANSPORT;
    }
    if(xmlnode_get_attrib(x, "etherx:error") != NULL)
        e->type = EPACKET_ERROR;

    e->ns = pstrdup(xmlnode_pool(x), ns);
    e->x = x;
    return e;
}

void epacket_error(epacket e, char *error)
{
    char *med;

    if(e == NULL)
        return;

    /* reverse addresses */
    med = e->to;
    e->to = e->from;
    e->from = med;

    e->type = EPACKET_ERROR;
    e->error = pstrdup(xmlnode_pool(e->x), error);
}