/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <jabber/jlib.h>
#include "etherx.h"

/* code to abstract the sending of epackets to etherx either via memory or sockets */

/* from socket.c */
extern thread _etherx_thread;
extern void (*_etherx_receive_handler)(epacket e);
extern char *_etherx_namespace;

void (*_etherx_send_handler)(epacket e) = NULL;
void (*_etherx_alias_handler)(void (*handler)(epacket e), char *host, char *namespace);

void etherx_send(epacket e)
{

    if(_etherx_thread != NULL)
    { /* use the socket */
        jdebug(ZONE,"etherx_send: socket");
        xmlnode_put_attrib(e->x, "etherx:to", e->to);
        xmlnode_put_attrib(e->x, "etherx:from", e->from);
        xmlstream_write(_etherx_thread, e->x);
    }
    if(_etherx_send_handler != NULL)
    { /* we're dso'd in the same process, send to etherxd */
        jdebug(ZONE,"etherx_send: dso");
        e->ns = pstrdup(xmlnode_pool(e->x), _etherx_namespace); /* set the transports namespace */
        (*_etherx_send_handler)(e);
    }

    /* XXX if it's socket and etherx goes offline, the transport might still be dumping some packets to us, we should add and error and dump them right back so they don't get totally lost */
}

void etherx_alias(char *host)
{
    xmlnode x;

    if(host == NULL)
        return;

    if(_etherx_thread != NULL)
    { /* use the socket */
        jdebug(ZONE,"etherx_alias: socket");
        x = xmlnode_new_tag("etherx:alias");
        xmlnode_insert_cdata(x, host, strlen(host));
        xmlstream_write(_etherx_thread, x);
    }else{ /* we're dso'd in the same process, send to etherxd */
        jdebug(ZONE,"etherx_alias: dso");
        (*_etherx_alias_handler)(_etherx_receive_handler, host, _etherx_namespace);
    }
}

/* called within dso by etherxd to set etherxd handlers/data */
void _etherx_setup(void *send, void *receive, void *alias, char *namespace)
{
    _etherx_send_handler = send;
    _etherx_receive_handler = receive;
    _etherx_alias_handler = alias;
    _etherx_namespace = j_strdup(namespace);
}
