/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <jabber/jlib.h>
#include "etherx.h"

thread _etherx_thread = NULL;
void (*_etherx_receive_handler)(epacket e);
char *_etherx_namespace = NULL;
int _etherx_retry = 0;

/* only called by socket connections, starts xmlstream to etherxd, returns success */
int etherx_init(char *id, char *secret, char *host, char *namespace, void (*receive)(epacket e))
{
    xmlnode x;

    /* so that etherx_init can be rerun internally and it will remember the orig args from the app */
    static char *st_id = NULL;
    static char *st_secret = NULL;
    static char *st_host = NULL;

    if(st_id == NULL)
        st_id = j_strdup(id);
    if(st_secret == NULL)
        st_secret = j_strdup(secret);
    if(st_host == NULL)
        st_host = j_strdup(host);
    if(st_host == NULL)
        st_host = j_strdup("localhost");

    if(namespace != NULL)
        _etherx_namespace = j_strdup(namespace);

    if(receive != NULL)
        _etherx_receive_handler = receive;

    _etherx_thread = xmlstream_connect(5269, st_host, (xmlstream_onNode)_etherx_receive);
    if(_etherx_thread <= 0)
        return -1;

    jdebug(ZONE,"_etherx_thread[%d] (new)\n",_etherx_thread);
    x = xmlstream_header(_etherx_namespace, st_host, NULL);
    xmlnode_put_attrib(x, "xmlns:etherx", ETHERX_NAMESPACE);
    xmlstream_write_header(_etherx_thread, x);

    x = xmlnode_new_tag("etherx:transport");
    xmlnode_put_attrib(x, "secret", st_secret);
    xmlnode_insert_cdata(x, st_id, strlen(st_id));
    xmlstream_write(_etherx_thread,x);

    x = xmlnode_new_tag("etherx:registry");
    xmlnode_put_attrib(x, "type", "request");
    xmlstream_write(_etherx_thread,x);

    if(receive != NULL)
        (*receive)(NULL); /* tell the transport to start up the first time */

    return 1;
}

void _etherx_receive(thread t, xmlnode x, int flag)
{
    epacket e;

    switch(flag)
    {
        case XMLSTREAM_ROOT:
            jdebug(ZONE,"got root tag from etherx!");
            /* ignore the root tag, transports don't use it */
            break;
        case XMLSTREAM_BOUNCE:
            e = epacket_new(xmlnode_dup(x), NULL, NULL, NULL);
            jerror("etherxd","bouncing data intended for etherx");
            epacket_error(e,"delivery failure");
            (*_etherx_receive_handler)(e);
        case XMLSTREAM_NODE:
            jdebug(ZONE,"got node from etherx!");
            if(j_strcmp(xmlnode_get_name(x),"etherx:transport") == 0)
            { /* if we successfully get connected */
                _etherx_retry = 1;
                return;
            }
            e = epacket_new(xmlnode_dup(x), NULL, NULL, NULL);
            (*_etherx_receive_handler)(e);
            break;
        case XMLSTREAM_INIT:
            pth_cleanup_push(_etherx_cleanup, (void *)t);
    }
}

void _etherx_cleanup(void *arg)
{
    if(_etherx_retry)
    {
        _etherx_thread = NULL;
        _etherx_retry = 0;
        jwarn("libetherx","Connection to etherxd failed, attempting to reconnect...");
        sleep(2); /* give etherxd a chance to restart if it was killed */
        /* the etherx_alias() need to be stored and resent if we get reconnected! */
        if(etherx_init(NULL, NULL, NULL, NULL, NULL) > 0)
            return;
    }

     jerror("libetherx","unable to estabilsh connection to etherx, forcing exit");
     raise(SIGTERM);
}
